#include "Linija.h"

Linija::Linija(int kap) : kap_(kap), pop_(0) {
	// Kada se kreira niz objekata neke klase, tada se svaki od elemenata tog niza (svaki objekat Tacke) inicijalizuje podrazumevanim konstruktorom
	// Ukoliko takav konstruktor ne postoji, prevodilac prijavljuje gresku u prevodjenju
	niz_ = new Tacka[kap_];
}

/*
	Kopirajuci konstruktor treba da vrsi duboku kopiju svih polja objekta.
	Duboka kopija kopira polja prostih tipova na uobicajen nacin, polja pokazivackih tipova tako sto kopira !!!objekat na koji pokazuju!!!, a ne pokazivac,
	i polja slozenih tipova tako sto pozove njihov kopirajuci konstruktor.
*/
Linija::Linija(const Linija& druga) {
	pop_ = druga.pop_;
	kap_ = druga.kap_;
	// Kreiramo nov niz da bismo dobili fizicki odvojene kopije
	niz_ = new Tacka[kap_];
	// A zatim kopiramo elemente niza
	for (int i = 0; i < pop_; i++) {
		niz_[i] = druga.niz_[i];
	}
}

/*
	Premestajuci konstruktor sluzi da optimizuje performanse.
	Zasto bismo kopirali sve elemente niza (moze mnogo da ih bude) ako ce vec drugi objekat da nestane?
	Premestajuci konstruktor 'preuzima' sva polja drugog objekta ukljucujuci i pokazivacka a samim tim i deo u dinamickoj memoriji da ne bismo vrsili nepotrebnu kopiju.
	Premestajuci konstruktor treba da vrsi plitku kopiju svih polja objekta.
	Nakon toga, treba !!!razvezati pokazivacka polja drugog objekta!!! od tekuceg objekta i postaviti ih na 'nullptr'
*/
Linija::Linija(Linija&& druga) {
	// Kopiramo sva polja jedno po jedno
	kap_ = druga.kap_;
	pop_ = druga.pop_;
	niz_ = druga.niz_;
	// Razvezemo pokazivacka polja druge linije
	// Zasto?
	// druga linija ce sigurno uskoro nestati jer je u pitanju samo privremeni rezultat u izrazu
	// U trenutku kad nestane, pozvace se njen destruktor koji ce unistiti niz u dinamickoj memoriji
	// Da ne bismo unistili niz naseg objekta, moramo pokazivac na niz drugog objekta da vratimo na nullptr
	druga.niz_ = nullptr;
}

Linija::~Linija() {
	delete[] niz_;
	niz_ = nullptr; kap_ = pop_ = 0;
}

void Linija::dodajTacku(const Tacka& t) {
	if (pop_ == kap_) return;
	niz_[pop_++] = t;
}

void Linija::pisi() const {
	for (int i = 0; i < pop_; i++) {
		niz_[i].pisi(); cout << endl;
	}
}
