#include "Tacka.h"
#include "Linija.h"

// && - prvalue - cisto desna vrednost
Linija vratiPriveremenuLiniju(Linija&& linija) { return linija; }

int main() {

	Tacka t1(10, -10);
	Tacka t2;
	Tacka t3(5);
	Tacka t4(t1);

	// Klasa Linija nije bezbedna za koriscenje jer sadrzi pokazivacka polja i koristi ugradjeni kopirajuci i premestajuci konstruktor
	Linija linija(4);

	linija.dodajTacku(t1);
	linija.dodajTacku(t2);
	linija.dodajTacku(t3);

	linija.pisi(); cout << endl;

	// Prilikom kreiranja novog objekta, pokazivacko polje 'niz_' se kopiralo iz 'linija' u 'linijaCpy'
	// Iz tog razloga oba objekta pokazuju na isti niz u dinamickoj memoriji => oba objekta koriste isti niz pa ce promene nad tim nizom biti vidljive u oba objekta
	// (hteli smo stvarnu kopiju objekta 'linija' a dobili smo delimicnu zavisnost izmedju ova dva objekta)
	Linija linijaCpy(linija);
	linijaCpy.pisi(); cout << endl;

	cout << "------------------------------" << endl;
	// Sta se desava sa 'linijaCpy' ako koristimo kopirajucu dodelu? Koji problemi postoje

	linija.dodajTacku(t4);			// Obe linije ce imati tacku t4 u svom nizu
	linija.pisi(); cout << endl;	// Ispisace se sve 4 tacke u nizu jer je linija.pop_ == 4
	linijaCpy.pisi(); cout << endl;	// Ispisace se 3 tacke u nizu jer je linijaCpy.pop_ i dalje == 3 iako u nizu postoji i 4. tacka

	cout << "------------------------------" << endl;

	linijaCpy.dodajTacku(Tacka(-9, -9)); // Kako je linijaCpy.pop_ == 3, novododata tacka ce pregaziti tacku koju smo dodali u objekat 'linija'
	linija.pisi(); cout << endl;		 // Za obe linije ce se ispisati sve 4 tacke, pri cemu je 4. tacka (9, -9) sto je nekorektno za objekat 'linija'
	linijaCpy.pisi(); cout << endl;		 // Razlog je taj sto objekti 'linija' i 'linijaCpy' barataju nad istim nizom u dinamickoj memoriji

	cout << "------------------------------" << endl;
	// Sta se desava ako iskoristimo premestajucu dodelu? Koji je tu problem?

	Linija linijaX(vratiPriveremenuLiniju(Linija(4)));	// Vraca se privremena linija koja ce nestati pa se objekat inicijalizuje premestajucom inicijalizacijom
	// Odmah nakon zavrsavanja gornjeg izraza privremena linija ce nestati pozivom njenog destruktora. Samim tim niz na koji ona ukazuje ce nestati. 
	// Kako i 'linijaX' ukazuje na isti taj niz jer su se sami pokazivaci kopirali, i njen niz ce automatski biti unisten
	
	linijaX.dodajTacku(Tacka(1, 1)); // Ne postoji niz u koji mozemo da dodamo tacku. Niz je izbrisan. Ova linija bi uglavnom prosla, ali bi pregazila deo memorije koji je slobodan i moze da se dodeli nekome drugom.

	return 0;
}

// Nakon zivotnog veka objekata 'linija' i 'linijaCpy' pozivaju se njihovi destruktori
// Destruktor objekta 'linija' ce unistiti i odgovaraju niz u dinamickoj memoriji
// Destruktor objekta 'linijaCpy' ce pokusati da unisti niz u dinamickoj memoriji, medjutim on je vec unisten i odgovarajuci pokazivac 'niz_' ukazuje na prazan deo memorije
// operator delete[] koji pozivamo u destruktoru (Linija.cpp - line 10) ce javiti gresku u izvrsavanju programa jer nece uspeti ponovo da unisti vec unisten niz
// Problem je u tome sto su oba objekta sadrzala isti niz pa je unistavanjem jednog objekta niz automatski unisten i za drugi objekat