#include "Linija.h"

/*
	Kopirajuci konstruktor treba da vrsi duboku kopiju svih polja objekta.
	Duboka kopija kopira polja prostih tipova na uobicajen nacin, polja pokazivackih tipova tako sto kopira objekat na koji pokazuju, a ne pokazivac,
	i polja slozenih tipova tako sto pozove njihov kopirajuci konstruktor.
*/
Linija::Linija(const Linija& druga) {
	// Napredna 'for' petlja. Dohvata jedan po jedan element iz kolekcije.
	// Tip promenljive 't' se moze eksplicitno navesti a moze se i automatski odrediti u vreme prevodjenja navodjenjem kljucne reci 'auto' umesto imena tipa, a na osnovu tipa elementa kolekcije.
	for (auto t : druga.niz_) {
		// S obzirom da nemamo mogucnost kopiranja tacaka, moramo kreirati novu kopiju koju inicijalizujemo vrednostima polja druge tacke.
		Tacka* tmp = new Tacka(t->dohvatiX(), t->dohvatiY());
		niz_.push_back(tmp);
	}
}

/*
	Premestajuci konstruktor treba da vrsi plitku kopiju svih polja objekta.
	Nakon toga, treba razvezati pokazivacka polja drugog objekta od tekuceg objekta i postaviti ih na 'nullptr'
*/
Linija::Linija(Linija&& druga) {
	// .begin() funckija dohvata 'iterator' na prvi element kolekcije
	// .end() funkcija dohvata 'iterator' na prvi element koji bi se nasao nakon poslednjeg elementa u kolekciji
	// iteratori se ponasaju kao pokazivaci na elemente kolekcije: mozemo ih inkrementirati da pokazuju na sledeci element (it++),
	//															   mozemo ih uporedjivati sa drugim iteratorima (it != druga.niz_.end() )
	//															   mozemo dohvatati ono na sta pokazuju - element kolekcije = pokazivac na Tacku (*it)
	for (auto it = druga.niz_.begin(); it != druga.niz_.end(); it++) {
		// dovoljno je da kopiramo sve elemente kolekcije plitko
		niz_.push_back(*it);
		// odgovarajuca pokazivacka polja kolekcije druge linije razvezujemo od nasih polja
		*it = nullptr;
	}
}

Linija::~Linija() {
	for (auto t : niz_) {
		delete t;
	}
	/*
	for (auto it = niz_.begin(); it != niz_.end(); it++) {
		delete *it;
	}
	*/
}

void Linija::dodajTacku(const Tacka& t) {
	Tacka* tmp = new Tacka(t.dohvatiX(), t.dohvatiY());
	niz_.push_back(tmp);
}

void Linija::pisi() const {
	for (auto t : niz_) {
		t->pisi(); cout << endl;
	}
	/*
	for (auto it = niz_.begin(); it != niz_.end(); it++) {
		(*it)->pisi(); cout << endl;
	}
	*/
}
